@@ -17,7 +17,7 @@ parser = argparse.ArgumentParser(prog = 'litepipeline')
 
 # common arguments
 parser.add_argument("address", help = "manager address, host:port")
-parser.add_argument("-w", "--column_width", help = "column max width", type = int, default = 0)
+parser.add_argument("-W", "--column_width", help = "column max width", type = int, default = 0)
 parser.add_argument("-v", "--version", action = 'version', version = '%(prog)s ' + __version__)
 subparsers = parser.add_subparsers(dest = "object", help = 'sub-command help')
 
@@ -113,12 +113,82 @@ parser_workspace_download.add_argument("-n", "--name", required = True, help = "
 parser_workspace_download.add_argument("-f", "--force", help = "force repack workspace", action = "store_true")
 parser_workspace_download.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
 
+# operate with workflow
+parser_workflow = subparsers.add_parser("workflow", help = "operate with workflow API")
+subparsers_workflow = parser_workflow.add_subparsers(dest = "operation", help = 'sub-command workflow help')
+
+parser_workflow_create = subparsers_workflow.add_parser("create", help = "create workflow")
+parser_workflow_create.add_argument("-n", "--name", required = True, help = "workflow's name", default = "")
+parser_workflow_create.add_argument("-c", "--config", help = "workflow's json configuration file", default = "")
+parser_workflow_create.add_argument("-d", "--description", help = "workflow's description", default = "")
+parser_workflow_create.add_argument("-e", "--enable", choices = ["true", "false"], help = "workflow's enable flag", default = "false")
+parser_workflow_create.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
+parser_workflow_delete = subparsers_workflow.add_parser("delete", help = "delete workflow")
+parser_workflow_delete.add_argument("-w", "--workflow_id", required = True, help = "workflow id", default = "")
+parser_workflow_delete.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
+parser_workflow_update = subparsers_workflow.add_parser("update", help = "update workflow")
+parser_workflow_update.add_argument("-w", "--workflow_id", required = True, help = "workflow id", default = "")
+parser_workflow_update.add_argument("-n", "--name", help = "workflow's name", default = "")
+parser_workflow_update.add_argument("-c", "--config", help = "workflow's json configuration file", default = "")
+parser_workflow_update.add_argument("-d", "--description", help = "workflow's description", default = "")
+parser_workflow_update.add_argument("-e", "--enable", choices = ["true", "false"], help = "workflow's enable flag", default = "false")
+parser_workflow_update.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
+parser_workflow_list = subparsers_workflow.add_parser("list", help = "list workflow")
+parser_workflow_list.add_argument("-o", "--offset", help = "list offset", type = int, default = 0)
+parser_workflow_list.add_argument("-l", "--limit", help = "list limit", type = int, default = 0)
+parser_workflow_list.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
+parser_workflow_info = subparsers_workflow.add_parser("info", help = "workflow's info")
+parser_workflow_info.add_argument("-w", "--workflow_id", required = True, help = "workflow id", default = "")
+parser_workflow_info.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
+# operate with work
+parser_work = subparsers.add_parser("work", help = "operate with work API")
+subparsers_work = parser_work.add_subparsers(dest = "operation", help = 'sub-command work help')
+
+parser_work_create = subparsers_work.add_parser("create", help = "create work")
+parser_work_create.add_argument("-w", "--workflow_id", required = True, help = "workflow id", default = "")
+parser_work_create.add_argument("-n", "--name", required = True, help = "work's name", default = "")
+parser_work_create.add_argument("-i", "--input", help = "work's input data, json string", default = "{}")
+parser_work_create.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
+parser_work_delete = subparsers_work.add_parser("delete", help = "delete work")
+parser_work_delete.add_argument("-w", "--work_id", required = True, help = "work id", default = "")
+parser_work_delete.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
+parser_work_list = subparsers_work.add_parser("list", help = "list works")
+parser_work_list.add_argument("-o", "--offset", help = "list offset", type = int, default = 0)
+parser_work_list.add_argument("-l", "--limit", help = "list limit", type = int, default = 0)
+parser_work_list.add_argument("-s", "--stage", choices = ["pending", "running", "finished"], help = "task's executing stage", default = "")
+parser_work_list.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
+parser_work_info = subparsers_work.add_parser("info", help = "work's info")
+parser_work_info.add_argument("-w", "--work_id", required = True, help = "work id", default = "")
+parser_work_info.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
+parser_work_rerun = subparsers_work.add_parser("rerun", help = "rerun work")
+parser_work_rerun.add_argument("-w", "--work_id", required = True, help = "work id", default = "")
+parser_work_rerun.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
+parser_work_recover = subparsers_work.add_parser("recover", help = "recover work")
+parser_work_recover.add_argument("-w", "--work_id", required = True, help = "work id", default = "")
+parser_work_recover.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
+parser_work_stop = subparsers_work.add_parser("stop", help = "stop work")
+parser_work_stop.add_argument("-w", "--work_id", required = True, help = "work id", default = "")
+parser_work_stop.add_argument("-g", "--signal", help = "stop work's signal: -9 or -15, default is -9", type = int, default = -9)
+parser_work_stop.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
 # operate with schedule
 parser_schedule = subparsers.add_parser("schedule", help = "operate with schedule API")
 subparsers_schedule = parser_schedule.add_subparsers(dest = "operation", help = 'sub-command schedule help')
 
 parser_schedule_create = subparsers_schedule.add_parser("create", help = "create schedule")
-parser_schedule_create.add_argument("-a", "--app_id", required = True, help = "application id", default = "")
+parser_schedule_create.add_argument("-a", "--app_id", help = "application id", default = "")
+parser_schedule_create.add_argument("-w", "--workflow_id", help = "workflow id", default = "")
 parser_schedule_create.add_argument("-n", "--name", required = True, help = "schedule's name", default = "")
 parser_schedule_create.add_argument("-i", "--input", help = "task's input data, json string", default = "{}")
 parser_schedule_create.add_argument("-m", "--minute", help = "minute, [0, 59]", type = int, default = -1)
@@ -131,6 +201,7 @@ parser_schedule_create.add_argument("-r", "--raw", help = "display raw json data
 parser_schedule_update = subparsers_schedule.add_parser("update", help = "update schedule")
 parser_schedule_update.add_argument("-s", "--schedule_id", required = True, help = "schedule id", default = "")
 parser_schedule_update.add_argument("-a", "--app_id", help = "application id")
+parser_schedule_update.add_argument("-w", "--workflow_id", help = "workflow id")
 parser_schedule_update.add_argument("-n", "--name", help = "schedule's name")
 parser_schedule_update.add_argument("-i", "--input", help = "task's input data, json string")
 parser_schedule_update.add_argument("-m", "--minute", help = "minute, [0, 59]", type = int)
@@ -347,7 +418,7 @@ def main():
                         else:
                             print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
                     else:
-                        print("error: need app_id(-a, --app_id) parameter")    
+                        print("error: need app_id(-a, --app_id) parameter")
             elif object == "task":
                 if operation == "list":
                     url += "?offset=%s&limit=%s" % (args.offset, args.limit)
@@ -627,6 +698,299 @@ def main():
                             print(e)
                     else:
                         parser.print_help()
+            elif object == "workflow":
+                if operation == "list":
+                    url += "?offset=%s&limit=%s" % (args.offset, args.limit)
+                    r = requests.get(url)
+                    if r.status_code == 200:
+                        data = r.json()
+                        if raw:
+                            print(json.dumps(data, indent = 4, sort_keys = True))
+                        else:
+                            if data["result"] == "ok": 
+                                print_table_result(
+                                    data["workflows"],
+                                    [
+                                        "workflow_id",
+                                        "name",
+                                        "create_at",
+                                        "update_at",
+                                    ]
+                                )
+                            else:
+                                print_table_result(
+                                    [data],
+                                    ["result", "message"]
+                                )
+                    else:
+                        print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                elif operation == "info":
+                    if args.workflow_id:
+                        url += "?workflow_id=%s" % args.workflow_id
+                        r = requests.get(url)
+                        if r.status_code == 200:
+                            data = r.json()
+                            if raw:
+                                print(json.dumps(data, indent = 4, sort_keys = True))
+                            else:
+                                if data["result"] == "ok":
+                                    print_table_result(
+                                        [data["info"]],
+                                        [
+                                            "workflow_id",
+                                            "name",
+                                            "create_at",
+                                            "update_at",
+                                            "description",
+                                        ]
+                                    )
+                                else:
+                                    print_table_result(
+                                        [data],
+                                        ["result", "message"]
+                                    )
+                        else:
+                            print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                    else:
+                        parser.print_help()
+                elif operation == "delete":
+                    if args.workflow_id:
+                        url += "?workflow_id=%s" % args.workflow_id
+                        r = requests.delete(url)
+                        if r.status_code == 200:
+                            data = r.json()
+                            if raw:
+                                print(json.dumps(data, indent = 4, sort_keys = True))
+                            else:
+                                print_table_result(
+                                    [data],
+                                    ["result", "message"]
+                                )
+                        else:
+                            print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                    else:
+                        parser.print_help()
+                elif operation == "create":
+                    if args.name:
+                        data = {
+                        	"name": args.name,
+                        	"configuration": {},
+                        	"description": args.description,
+                        	"enable": True if args.enable == "true" else False,
+                        }
+                        if args.config and os.path.exists(args.config) and os.path.isfile(args.config):
+                            fp = open(args.config, "r")
+                            content = fp.read()
+                            fp.close()
+                            if content:
+                                data["configuration"] = json.loads(content)
+
+                        r = requests.post(url, json = data)
+                        if r.status_code == 200:
+                            data = r.json()
+                            if raw:
+                                print(json.dumps(data, indent = 4, sort_keys = True))
+                            else:
+                                print_table_result(
+                                    [data],
+                                    ["workflow_id", "result", "message"]
+                                )
+                        else:
+                            print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                    else:
+                        parser.print_help()
+                elif operation == "update":
+                    if args.workflow_id:
+                        data = {}
+                        if args.config and os.path.exists(args.config) and os.path.isfile(args.config):
+                            fp = open(args.config, "r")
+                            content = fp.read()
+                            fp.close()
+                            if content:
+                                data["configuration"] = json.loads(content)
+                        if args.name:
+                            data["name"] = args.name
+                        if args.description:
+                            data["description"] = args.description
+                        if args.enable is not None:
+                            data["enable"] = True if args.enable == "true" else False
+                        if data:
+                            data["workflow_id"] = args.workflow_id
+                            r = requests.put(url, json = data)
+                            if r.status_code == 200:
+                                data = r.json()
+                                if raw:
+                                    print(json.dumps(data, indent = 4, sort_keys = True))
+                                else:
+                                    print_table_result(
+                                        [data],
+                                        ["workflow_id", "result", "message"]
+                                    )
+                            else:
+                                print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                    else:
+                        print("error: need app_id(-a, --app_id) parameter")
+            elif object == "work":
+                if operation == "list":
+                    url += "?offset=%s&limit=%s" % (args.offset, args.limit)
+                    if args.stage:
+                        url += "&stage=%s" % args.stage
+                    r = requests.get(url)
+                    if r.status_code == 200:
+                        data = r.json()
+                        if raw:
+                            print(json.dumps(data, indent = 4, sort_keys = True))
+                        else:
+                            if data["result"] == "ok":
+                                print_table_result(
+                                    data["works"],
+                                    [
+                                        "work_id",
+                                        "workflow_id",
+                                        "name",
+                                        "create_at",
+                                        "start_at",
+                                        "end_at",
+                                        "stage",
+                                        "status",
+                                    ]
+                                )
+                            else:
+                                print_table_result(
+                                    [data],
+                                    ["result", "message"]
+                                )
+                    else:
+                        print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                elif operation == "info":
+                    if args.work_id:
+                        url += "?work_id=%s" % args.work_id
+                        r = requests.get(url)
+                        if r.status_code == 200:
+                            data = r.json()
+                            if raw:
+                                print(json.dumps(data, indent = 4, sort_keys = True))
+                            else:
+                                if data["result"] == "ok":
+                                    print_table_result(
+                                        [data["info"]],
+                                        [
+                                            "work_id",
+                                            "workflow_id",
+                                            "name",
+                                            "create_at",
+                                            "start_at",
+                                            "end_at",
+                                            "stage",
+                                            "status",
+                                        ]
+                                    )
+                                else:
+                                    print_table_result(
+                                        [data],
+                                        ["result", "message"]
+                                    )
+                        else:
+                            print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                    else:
+                        parser.print_help()
+                elif operation == "delete":
+                    if args.work_id:
+                        url += "?work_id=%s" % args.work_id
+                        r = requests.delete(url)
+                        if r.status_code == 200:
+                            data = r.json()
+                            if raw:
+                                print(json.dumps(data, indent = 4, sort_keys = True))
+                            else:
+                                print_table_result(
+                                    [data],
+                                    ["result", "message"]
+                                )
+                        else:
+                            print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                    else:
+                        parser.print_help()
+                elif operation == "create":
+                    if args.name and args.workflow_id and args.input:
+                        try:
+                            data = {"name": args.name, "workflow_id": args.workflow_id, "input_data": json.loads(args.input)}
+                            r = requests.post(url, json = data)
+                            if r.status_code == 200:
+                                data = r.json()
+                                if raw:
+                                    print(json.dumps(data, indent = 4, sort_keys = True))
+                                else:
+                                    print_table_result(
+                                        [data],
+                                        ["work_id", "result", "message"]
+                                    )
+                            else:
+                                print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                        except Exception as e:
+                            print(e)
+                    else:
+                        parser.print_help()
+                elif operation == "rerun":
+                    if args.work_id:
+                        try:
+                            data = {"work_id": args.work_id}
+                            r = requests.put(url, json = data)
+                            if r.status_code == 200:
+                                data = r.json()
+                                if raw:
+                                    print(json.dumps(data, indent = 4, sort_keys = True))
+                                else:
+                                    print_table_result(
+                                        [data],
+                                        ["result", "message"]
+                                    )
+                            else:
+                                print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                        except Exception as e:
+                            print(e)
+                    else:
+                        parser.print_help()
+                elif operation == "recover":
+                    if args.work_id:
+                        try:
+                            data = {"work_id": args.work_id}
+                            r = requests.put(url, json = data)
+                            if r.status_code == 200:
+                                data = r.json()
+                                if raw:
+                                    print(json.dumps(data, indent = 4, sort_keys = True))
+                                else:
+                                    print_table_result(
+                                        [data],
+                                        ["result", "message"]
+                                    )
+                            else:
+                                print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                        except Exception as e:
+                            print(e)
+                    else:
+                        parser.print_help()
+                elif operation == "stop":
+                    if args.work_id and args.signal:
+                        try:
+                            data = {"work_id": args.work_id, "signal": args.signal}
+                            r = requests.put(url, json = data)
+                            if r.status_code == 200:
+                                data = r.json()
+                                if raw:
+                                    print(json.dumps(data, indent = 4, sort_keys = True))
+                                else:
+                                    print_table_result(
+                                        [data],
+                                        ["result", "message"]
+                                    )
+                            else:
+                                print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                        except Exception as e:
+                            print(e)
+                    else:
+                        parser.print_help()
             elif object == "schedule":
                 if operation == "list":
                     url += "?offset=%s&limit=%s" % (args.offset, args.limit)
@@ -643,7 +1007,8 @@ def main():
                                     data["schedules"],
                                     [
                                         "schedule_id",
-                                        "application_id",
+                                        "source",
+                                        "source_id",
                                         "schedule_name",
                                         "create_at",
                                         "update_at",          
@@ -675,7 +1040,8 @@ def main():
                                         [data["schedule_info"]],
                                         [
                                             "schedule_id",
-                                            "application_id",
+                                            "source",
+                                            "source_id",
                                             "schedule_name",
                                             "create_at",
                                             "update_at",
@@ -713,11 +1079,10 @@ def main():
                     else:
                         parser.print_help()
                 elif operation == "create":
-                    if args.name and args.app_id and args.input:
+                    if args.name and args.input and (args.app_id or args.workflow_id):
                         try:
                             data = {
                                 "schedule_name": args.name,
-                                "app_id": args.app_id,
                                 "input_data": json.loads(args.input),
                                 "minute": args.minute,
                                 "hour": args.hour,
@@ -725,18 +1090,27 @@ def main():
                                 "day_of_week": args.day_of_week,
                                 "enable": True if args.enable == "true" else False,
                             }
-                            r = requests.post(url, json = data)
-                            if r.status_code == 200:
-                                data = r.json()
-                                if raw:
-                                    print(json.dumps(data, indent = 4, sort_keys = True))
+                            if args.app_id:
+                                data["source"] = "application"
+                                data["source_id"] = args.app_id
+                            elif args.workflow_id:
+                                data["source"] = "workflow"
+                                data["source_id"] = args.workflow_id
+                            if data["source"]:
+                                r = requests.post(url, json = data)
+                                if r.status_code == 200:
+                                    data = r.json()
+                                    if raw:
+                                        print(json.dumps(data, indent = 4, sort_keys = True))
+                                    else:
+                                        print_table_result(
+                                            [data],
+                                            ["schedule_id", "result", "message"]
+                                        )
                                 else:
-                                    print_table_result(
-                                        [data],
-                                        ["schedule_id", "result", "message"]
-                                    )
+                                    print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
                             else:
-                                print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                                parser.print_help()
                         except Exception as e:
                             print(e)
                     else:
@@ -748,7 +1122,11 @@ def main():
                             if args.name is not None:
                                 data["schedule_name"] = args.name
                             if args.app_id is not None:
-                                data["app_id"] = args.app_id
+                                data["source"] = "application"
+                                data["source_id"] = args.app_id
+                            elif args.workflow_id is not None:
+                                data["source"] = "workflow"
+                                data["source_id"] = args.workflow_id
                             if args.input is not None:
                                 data["input_data"] = args.input
                             if args.minute is not None:
